
/* ////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
//#include <stlsoft/stlsoft.h>

#define STLSOFT_SUPPRESS_UNUSED(x)      ((void)x)

/* ////////////////////////////////////////////////////////////////////// */

//#include <stlsoft/meta/yesno.hpp>

namespace stlsoft
{

    struct yes_type
    {
        char ar[32];
    };

    struct no_type
    {
        char ar[1];
    };


    template <bool B>
    struct value_to_yesno_type
    {
        enum { value = 1 };

        typedef yes_type    type;
    };

    template <>
    struct value_to_yesno_type<0>
    {
        enum { value = 0 };

        typedef no_type type;
    };

} // namespace stlsoft


/* ////////////////////////////////////////////////////////////////////// */

//#include <stlsoft/meta/is_same_type.hpp>

namespace stlsoft
{

    template<   typename T1
            ,   typename T2
            >
    struct is_same_type
    {
        enum { value = 0 };

        typedef no_type     type;
    };

    template <typename T>
    struct is_same_type<T, T>
    {
        enum { value = 1 };

        typedef yes_type    type;
    };

} // namespace stlsoft

/* /////////////////////////////////////////////////////////////////////////
 * Class definitions
 */

struct X
{};

class C1
    : public X
{
public:
    operator int() const
    {
        return 0;
    }

    int get() const
    {
        return 0;
    }
};

class C2
{
public:
    int get() const
    {
        return 0;
    }
};


template <typename T3>
inline int get_int_X(T3 const &t3, X const &)
{
    return t3.get();
}

template <typename T2>
inline int get_int(T2 const &c1)
{
    return get_int_X(c1, c1);
}

inline int get_int(C2 const &c2)
{
    return c2.get();
}

inline int get_int(int i)
{
    return i;
}

/* /////////////////////////////////////////////////////////////////////////
 * Globals
 */


inline void verify_non_int_type_has_shim_function(int, stlsoft::yes_type)
{}

template <typename T>
inline void verify_non_int_type_has_shim_function(T const &, stlsoft::no_type)
{
    int (*pfn)(T const&)    =   &get_int;

    STLSOFT_SUPPRESS_UNUSED(pfn);
}


template <typename T>
inline void f(T const &t)
{
    typedef typename stlsoft::is_same_type<T, int>::type    yesno_t;

    verify_non_int_type_has_shim_function(t, yesno_t());

    int i   =   get_int(t);

    STLSOFT_SUPPRESS_UNUSED(i);
}

/* /////////////////////////////////////////////////////////////////////////
 * Main
 */

int main()
{
    C1  c1;
    C2  c2;
    int i3 = 0;

    f(c1);
    f(c2);
    f(i3);

    return 0;
}

/* ////////////////////////////////////////////////////////////////////// */
